/*-
 * Copyright (c) 2002, 2003 Tim J. Robbins.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__FBSDID("$FreeBSD: ports/emulators/pearpc/files/wcrtomb.c,v 1.1 2004/05/31 14:02:28 pav Exp $");

#include <errno.h>
#include <limits.h>
#include <rune.h>
#include <stdlib.h>
#include <wchar.h>

/*
 * Emulate the ISO C wcrtomb() function in terms of the deprecated
 * 4.4BSD sputrune() function.
 */
size_t
wcrtomb(char * __restrict s, wchar_t wc,
    mbstate_t * __restrict ps __unused)
{
	char *e;
	char buf[MB_LEN_MAX];

	if (s == NULL) {
		s = buf;
		wc = L'\0';
	}
	sputrune(wc, s, MB_CUR_MAX, &e);
	if (e == NULL) {
		errno = EILSEQ;
		return ((size_t)-1);
	}
	return ((size_t)(e - s));
}
